@@ -1,11 +1,8 @@
 import logging
-import warnings
-
 import py2neo
-
-from enhanced_phpjoern_framework import Neo4jEngine
+from pjscan import AnalysisFramework
+from pjscan.const import *
 from cpg2code.three_address_code_path import ThreeAddressCodePath, ThreeAddressCode
-from enhanced_phpjoern_framework.const import *
 
 from html import unescape
 
@@ -91,12 +88,12 @@ class SymbolicTracking(object):
         """
         ABS_FLAG_ = 1
 
-    def __init__(self, _graph: Neo4jEngine):
+    def __init__(self, analysis_framework: AnalysisFramework):
         """
-        :param _graph: neo4j graph
+        :param analysis_framework: neo4j graph
         """
         self.tac_path = ThreeAddressCodePath()
-        self.neo4j_engine = _graph
+        self.analysis_framework = analysis_framework
         self.__global_abstract_level = 0b00000000
 
     def to_src_code(self):
@@ -128,28 +125,27 @@ class SymbolicTracking(object):
         return str(result)
 
     def get_expression_ith_node(self, ith: int) -> py2neo.Node:
-        node = self.neo4j_engine.match(
+        node = self.analysis_framework.match(
             code="_expr_{}".format(ith)
         ).first()
-        return self.neo4j_engine.get_ast_root_node(node)
+        return self.analysis_framework.get_ast_root_node(node)
 
     def get_condition_ith_node(self, ith: int) -> py2neo.Node:
-        node = self.neo4j_engine.match(
+        node = self.analysis_framework.match(
             code="_cond_{}".format(ith)
         ).first()
-        return self.neo4j_engine.get_ast_ith_child_node(
-            self.neo4j_engine.get_ast_root_node(node), 1
+        return self.analysis_framework.get_ast_ith_child_node(
+            self.analysis_framework.get_ast_root_node(node), 1
         )
 
     def clear_memory(self):
         self.tac_path = ThreeAddressCodePath()
-        self.neo4j_engine.clear_cache()
 
     def get_node(self, _id, ):
         """
         DEBUG FUNCTION
         """
-        return self.neo4j_engine.get_node_itself(_id)
+        return self.analysis_framework.get_node_itself(_id)
 
     def gen_formula(self, left: str, right: list, op: str, node_type: str, node_id: int, ltype: str, rtype: list):
         """
@@ -204,11 +200,12 @@ class SymbolicTracking(object):
             return self.manage_global_node(node)
 
     def manage_global_node(self, node: py2neo.Node):
-        return self.manage_generic_node(self.neo4j_engine.get_ast_child_node(node))
+        return self.manage_generic_node(self.analysis_framework.get_ast_child_node(node))
 
     def manage_type_static_prop(self, node: py2neo.Node):
-        class_code = self.neo4j_engine.get_ast_child_node(self.neo4j_engine.get_ast_child_node(node))['code']
-        expr_code = self.neo4j_engine.get_ast_ith_child_node(node, 1)['code']
+        class_code = self.analysis_framework.get_ast_child_node(self.analysis_framework.get_ast_child_node(node))[
+            'code']
+        expr_code = self.analysis_framework.get_ast_ith_child_node(node, 1)['code']
         return f"{class_code}::${expr_code}", "string"
         # return self.gen_formula(
         #     left=join_var_node(node.identity),
@@ -221,7 +218,7 @@ class SymbolicTracking(object):
         # )
 
     def manage_type_list(self, node: py2neo.Node):
-        list_nodes = self.neo4j_engine.find_ast_child_nodes(node)
+        list_nodes = self.analysis_framework.find_ast_child_nodes(node)
         resolved_list_nodes = []
         resolved_list_nodes_types = []
         for list_node in list_nodes:
@@ -241,10 +238,11 @@ class SymbolicTracking(object):
 
     def manage_instanceof_operation(self, node: py2neo.Node):
         judge_instance_var, judge_instance_var_type = self.manage_generic_node(
-            self.neo4j_engine.find_ast_child_nodes(node)[0])
+            self.analysis_framework.find_ast_child_nodes(node)[0])
 
         instance_var, instance_type = \
-            self.neo4j_engine.find_ast_child_nodes(self.neo4j_engine.find_ast_child_nodes(node)[1])[0]['code'], 'string'
+            self.analysis_framework.find_ast_child_nodes(self.analysis_framework.find_ast_child_nodes(node)[1])[0][
+                'code'], 'string'
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=[judge_instance_var, instance_var],
@@ -256,8 +254,8 @@ class SymbolicTracking(object):
         )
 
     def manage_new_operation(self, node: py2neo.Node):
-        args = self.neo4j_engine.find_function_arg_node_list(node)
-        name_node = self.neo4j_engine.find_ast_child_nodes(node)[0]
+        args = self.analysis_framework.ast_step.find_function_arg_node_list(node)
+        name_node = self.analysis_framework.find_ast_child_nodes(node)[0]
         resolved_args = []
         resolved_args_types = []
         for arg in args:
@@ -268,7 +266,7 @@ class SymbolicTracking(object):
         expr, expr_type = self.gen_formula(
             left=join_var_node(name_node.identity),
             right=resolved_args,
-            op=self.neo4j_engine.find_ast_child_nodes(name_node)[0]['code'],
+            op=self.analysis_framework.find_ast_child_nodes(name_node)[0][NODE_CODE],
             node_type=TYPE_CALL,
             node_id=node.identity,
             ltype="",  # object
@@ -285,7 +283,7 @@ class SymbolicTracking(object):
         )
 
     def manage_function_call(self, node: py2neo.Node):
-        args = self.neo4j_engine.find_function_arg_node_list(node)
+        args = self.analysis_framework.ast_step.find_function_arg_node_list(node)
         resolved_args = []
         resolved_args_types = []
         for arg in args:
@@ -296,7 +294,7 @@ class SymbolicTracking(object):
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=resolved_args,
-            op=self.neo4j_engine.get_ast_call_code(node),
+            op=self.analysis_framework.code_step.get_ast_call_code(node),
             node_type=TYPE_CALL,
             node_id=node.identity,
             ltype="string",
@@ -304,11 +302,11 @@ class SymbolicTracking(object):
         )
 
     def manage_exit_operation(self, node: py2neo.Node):
-        if self.neo4j_engine.find_ast_child_nodes(node).__len__() == 0:
+        if self.analysis_framework.find_ast_child_nodes(node).__len__() == 0:
             expr, expr_type = None, None
         else:
             expr, expr_type = self.manage_generic_node(
-                self.neo4j_engine.find_ast_child_nodes(node)[0]
+                self.analysis_framework.find_ast_child_nodes(node)[0]
             )
         return self.gen_formula(
             left=join_var_node(node.identity),
@@ -322,7 +320,7 @@ class SymbolicTracking(object):
 
     def manage_empty_operation(self, node: py2neo.Node):
         condition = self.manage_generic_node(
-            self.neo4j_engine.find_ast_child_nodes(node)[0]
+            self.analysis_framework.find_ast_child_nodes(node)[0]
         )
         return self.gen_formula(
             left=join_var_node(node.identity),
@@ -335,7 +333,7 @@ class SymbolicTracking(object):
         )
 
     def manage_var(self, node: py2neo.Node):
-        return self.neo4j_engine.get_ast_var_code(node), ""
+        return self.analysis_framework.code_step.get_ast_var_code(node), ""
 
     def manage_dim(self, node: py2neo.Node):
         """
@@ -343,9 +341,9 @@ class SymbolicTracking(object):
         =>
         get_slice($a,'shaobao'] / get_slice($a->file,'shaobao']
         """
-        nodes = self.neo4j_engine.find_ast_child_nodes(node)
+        nodes = self.analysis_framework.find_ast_child_nodes(node)
         if nodes[0]['type'] == TYPE_VAR and nodes[1]['type'] == TYPE_STRING:
-            return self.neo4j_engine.get_ast_dim_code(node), ""
+            return self.analysis_framework.code_step.get_ast_dim_code(node), ""
         else:
             expr_var, expr_type = self.manage_generic_node(nodes[0])
             slice_var, slice_type = self.manage_generic_node(nodes[1])
@@ -357,8 +355,8 @@ class SymbolicTracking(object):
                 return expr_var + '[' + str(slice_var) + ']', ""
 
     def manage_binary_operation(self, node: py2neo.Node):
-        l_expr = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
-        r_expr = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[1])
+        l_expr = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
+        r_expr = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[1])
         if node['flags'][0] in ["BINARY_CONCAT"]:
             return self.gen_formula(
                 left=join_var_node(node.identity),
@@ -423,7 +421,7 @@ class SymbolicTracking(object):
         elif node[NODE_TYPE] in {TYPE_ASSIGN, TYPE_ASSIGN_REF, TYPE_ASSIGN_OP, TYPE_STATIC}:
             return self.manage_assignment_operation(node)
         elif node[NODE_TYPE] == TYPE_EXPR_LIST:
-            return self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
+            return self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
         elif node[NODE_TYPE] == TYPE_ENCAPS_LIST:
             return self.manage_encapsulated_list(node)
             # 预定义函数的type
@@ -465,11 +463,11 @@ class SymbolicTracking(object):
         return '', ''
 
     def manage_param_operation(self, node: py2neo.Node):
-        if self.neo4j_engine.get_ast_ith_child_node(node, i=2)[NODE_TYPE] == TYPE_NULL:
+        if self.analysis_framework.get_ast_ith_child_node(node, i=2)[NODE_TYPE] == TYPE_NULL:
             return '', ''
         else:
-            var, var_type = self.manage_generic_node(self.neo4j_engine.get_ast_ith_child_node(node, i=1))
-            value, value_type = self.manage_generic_node(self.neo4j_engine.get_ast_ith_child_node(node, i=2))
+            var, var_type = self.manage_generic_node(self.analysis_framework.get_ast_ith_child_node(node, i=1))
+            value, value_type = self.manage_generic_node(self.analysis_framework.get_ast_ith_child_node(node, i=2))
             return self.gen_formula(left=var, right=[value], ltype=var_type, rtype=[value_type], op=TYPE_ASSIGN,
                                     node_type=TYPE_ASSIGN, node_id=node[NODE_INDEX], )
 
@@ -480,33 +478,33 @@ class SymbolicTracking(object):
         :param node:
         :return:
         """
-        a, a_type = self.manage_generic_node(self.neo4j_engine.get_ast_ith_child_node(node, i=0))
-        if self.neo4j_engine.get_ast_ith_child_node(node, i=2)[NODE_TYPE] == TYPE_NULL:
+        a, a_type = self.manage_generic_node(self.analysis_framework.get_ast_ith_child_node(node, i=0))
+        if self.analysis_framework.get_ast_ith_child_node(node, i=2)[NODE_TYPE] == TYPE_NULL:
             # CASE A
-            v, v_type = self.manage_generic_node(self.neo4j_engine.get_ast_ith_child_node(node, i=1))
+            v, v_type = self.manage_generic_node(self.analysis_framework.get_ast_ith_child_node(node, i=1))
             a, a_type = self.gen_formula(  # $a = $a[0] => $a = array_search(0,$a)
                 left=a, right=[0, a], ltype='', rtype=['number', a_type], op='array_search', node_type=TYPE_CALL,
                 node_id=node[NODE_INDEX]
             )
         else:
             # CASE A
-            k, k_type = self.manage_generic_node(self.neo4j_engine.get_ast_ith_child_node(node, i=1))
-            v, v_type = self.manage_generic_node(self.neo4j_engine.get_ast_ith_child_node(node, i=2))
+            k, k_type = self.manage_generic_node(self.analysis_framework.get_ast_ith_child_node(node, i=1))
+            v, v_type = self.manage_generic_node(self.analysis_framework.get_ast_ith_child_node(node, i=2))
             a, a_type = self.gen_formula(  # $a = $a[0] => $a = array_search(0,$a)
                 left=a, right=[k, a], ltype='', rtype=[k_type, a_type], op='array_search', node_type=TYPE_CALL,
                 node_id=node[NODE_INDEX]
             )
         return self.gen_formula(
             left=v, right=[a], ltype=v_type, rtype=[a_type], op=TYPE_ASSIGN, node_type=TYPE_ASSIGN,
-            node_id=self.neo4j_engine.get_ast_ith_child_node(node, i=1)[NODE_INDEX]
+            node_id=self.analysis_framework.get_ast_ith_child_node(node, i=1)[NODE_INDEX]
         )
 
     def manage_return_operation(self, node: py2neo.Node):
-        return self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
+        return self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
 
     def manage_condition_operation(self, node: py2neo.Node):
         # 三目运算符 别到时候碰到一个 ?? 我就吐了
-        _the_three = self.neo4j_engine.find_ast_child_nodes(node)
+        _the_three = self.analysis_framework.find_ast_child_nodes(node)
         condition_node, true_path_node, false_path_node = _the_three[0], _the_three[1], _the_three[2]
         condition_expr, condition_type = self.manage_generic_node(condition_node)
         true_expr, true_type = self.manage_generic_node(true_path_node)
@@ -526,7 +524,7 @@ class SymbolicTracking(object):
 
     def manage_ref(self, node: py2neo.Node):
         # we ignore it !
-        return self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
+        return self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
         # var, var_type = self.manage_generic_node(
         # )
         # return self.gen_formula(
@@ -587,7 +585,7 @@ class SymbolicTracking(object):
                 ltype="",
                 rtype=["STRING"]
             )
-        args = self.neo4j_engine.find_ast_child_nodes(node)  # child node's type is array elem
+        args = self.analysis_framework.find_ast_child_nodes(node)  # child node's type is array elem
         resolved_array_elem = []
         resolved_array_elem_types = []
         for arg in args:
@@ -607,14 +605,14 @@ class SymbolicTracking(object):
 
     def manage_type_array_elem(self, node: py2neo.Node):
         assert node[NODE_TYPE] == TYPE_ARRAY_ELEM
-        if self.neo4j_engine.get_ast_ith_child_node(node, 1)['type'] == 'NULL':
-            array_node = self.neo4j_engine.get_parent_node(node)
-            ith_node = self.neo4j_engine.find_ast_child_nodes(array_node).index(node)
+        if self.analysis_framework.get_ast_ith_child_node(node, 1)['type'] == 'NULL':
+            array_node = self.analysis_framework.ast_step.get_parent_node(node)
+            ith_node = self.analysis_framework.find_ast_child_nodes(array_node).index(node)
             key_expr, key_type = ith_node, 'number'
-            value_expr, value_type = self.manage_generic_node(self.neo4j_engine.get_ast_child_node(node))
+            value_expr, value_type = self.manage_generic_node(self.analysis_framework.get_ast_child_node(node))
         else:
-            key_expr, key_type = self.manage_generic_node(self.neo4j_engine.get_ast_ith_child_node(node, 1))
-            value_expr, value_type = self.manage_generic_node(self.neo4j_engine.get_ast_ith_child_node(node, 0))
+            key_expr, key_type = self.manage_generic_node(self.analysis_framework.get_ast_ith_child_node(node, 1))
+            value_expr, value_type = self.manage_generic_node(self.analysis_framework.get_ast_ith_child_node(node, 0))
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=[key_expr, value_expr],
@@ -626,7 +624,7 @@ class SymbolicTracking(object):
         )
 
     def manage_class_constant(self, node: py2neo.Node):
-        code = self.neo4j_engine.get_ast_class_const_code(node)
+        code = self.analysis_framework.code_step.get_ast_class_const_code(node)
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=[code],
@@ -641,7 +639,7 @@ class SymbolicTracking(object):
         return MAGIC_CONST_CONVERT_DICT[node['flags'][0]], "string"
 
     def manage_generic_constant(self, node: py2neo.Node):
-        code = self.neo4j_engine.get_ast_const_code(node)
+        code = self.analysis_framework.code_step.get_ast_const_code(node)
         if code.lower() == "true":
             return True, "boolean"
         elif code.lower() == "false":
@@ -658,7 +656,7 @@ class SymbolicTracking(object):
             )
 
     def manage_unary_operation(self, node: py2neo.Node):
-        expr, expr_type = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
+        expr, expr_type = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
         if node[NODE_TYPE] == 'AST_UNARY_OP' and set(node['flags']) & {FLAG_UNARY_SILENCE}:
             return expr, expr_type
         if node[NODE_TYPE] == 'AST_UNARY_OP' and set(node['flags']) & {"UNARY_BOOL_NOT"}:
@@ -709,8 +707,8 @@ class SymbolicTracking(object):
 
     def manage_assignment_operation(self, node: py2neo.Node):
         assert not set(node[NODE_TYPE]) & {TYPE_ASSIGN, TYPE_ASSIGN_REF, TYPE_ASSIGN_OP}
-        l_expr = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
-        r_expr = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[1])
+        l_expr = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
+        r_expr = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[1])
         if node[NODE_TYPE] == TYPE_ASSIGN_REF:
             return self.gen_formula(
                 left=l_expr[0], right=[r_expr[0]], op=TYPE_ASSIGN, node_type=TYPE_ASSIGN, node_id=node.identity,
@@ -757,7 +755,7 @@ class SymbolicTracking(object):
             raise Exception(node[NODE_TYPE])
 
     def manage_echo_operation(self, node: py2neo.Node):
-        expr = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
+        expr = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=[expr[0]],
@@ -769,7 +767,7 @@ class SymbolicTracking(object):
         )
 
     def manage_unset_operation(self, node: py2neo.Node):
-        expr = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
+        expr = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=[expr[0]],
@@ -781,7 +779,7 @@ class SymbolicTracking(object):
         )
 
     def manage_isset_operation(self, node: py2neo.Node):
-        expr = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
+        expr = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=[expr[0]],
@@ -794,7 +792,7 @@ class SymbolicTracking(object):
 
     def manage_encapsulated_list(self, node: py2neo.Node):
         pre_tmp = None
-        __nodes = self.neo4j_engine.find_ast_child_nodes(node)
+        __nodes = self.analysis_framework.find_ast_child_nodes(node)
         for __node in __nodes:
             expr = self.manage_generic_node(__node)
             if pre_tmp is None:
@@ -813,7 +811,7 @@ class SymbolicTracking(object):
         return pre_tmp, "string"
 
     def manage_static_function_call(self, node: py2neo.Node):
-        args = self.neo4j_engine.find_function_arg_node_list(node)
+        args = self.analysis_framework.ast_step.find_function_arg_node_list(node)
         resolved_args = []
         resolved_args_types = []
         for arg in args:
@@ -824,7 +822,7 @@ class SymbolicTracking(object):
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=resolved_args,
-            op=self.neo4j_engine.get_ast_static_call_code(node),
+            op=self.analysis_framework.code_step.get_static_call_code(node),
             node_type=TYPE_STATIC_CALL,
             node_id=node.identity,
             ltype="string",
@@ -838,7 +836,7 @@ class SymbolicTracking(object):
         R : func($a,$b,$c)
         特殊处理，将dy_func的第一位参数扩展为 $
         """
-        args = self.neo4j_engine.find_function_arg_node_list(node)
+        args = self.analysis_framework.ast_step.find_function_arg_node_list(node)
         resolved_args = []
         resolved_args_types = []
         for arg in args:
@@ -846,14 +844,14 @@ class SymbolicTracking(object):
             res_arg = self.manage_generic_node(arg)
             resolved_args.append(res_arg[0])
             resolved_args_types.append(res_arg[1])
-        var_node = self.neo4j_engine.get_ast_child_node(node)
+        var_node = self.analysis_framework.get_ast_child_node(node)
         var_expr, var_type = self.manage_generic_node(var_node)
         resolved_args = [var_expr] + resolved_args
         resolved_args_types = [var_type] + resolved_args_types
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=resolved_args,
-            op=self.neo4j_engine.get_ast_method_call_code(node),
+            op=self.analysis_framework.code_step.get_ast_method_call_code(node),
             node_type=TYPE_METHOD_CALL,
             node_id=node.identity,
             ltype="string",
@@ -861,7 +859,7 @@ class SymbolicTracking(object):
         )
 
     def manage_include_operation(self, node: py2neo.Node):
-        expr = self.manage_generic_node(self.neo4j_engine.find_ast_child_nodes(node)[0])
+        expr = self.manage_generic_node(self.analysis_framework.find_ast_child_nodes(node)[0])
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=[expr[0]],
@@ -873,7 +871,7 @@ class SymbolicTracking(object):
         )
 
     def manage_eval_operation(self, node: py2neo.Node):
-        expr = self.manage_generic_node(self.neo4j_engine.get_ast_child_node(node))
+        expr = self.manage_generic_node(self.analysis_framework.get_ast_child_node(node))
         return self.gen_formula(
             left=join_var_node(node.identity),
             right=[expr[0]],
@@ -893,7 +891,7 @@ class SymbolicTracking(object):
         """
         resolved_args = []
         resolved_args_types = []
-        args = self.neo4j_engine.find_ast_child_nodes(node)
+        args = self.analysis_framework.find_ast_child_nodes(node)
         for arg in args:
             assert isinstance(arg, py2neo.Node)
             res_arg = self.manage_generic_node(arg)
