@@ -2,7 +2,7 @@ import json
 import os
 import sys
 
-from enhanced_phpjoern_framework.const import *
+from pjscan.const import *
 
 import logging
 
@@ -99,7 +99,7 @@ class ThreeAddressCode(object):
 
     def to_navex_structure(self):
         """
-        转成NAVEX的形式，看看到底发生了什么
+
         """
         result = {}
         result["formula"] = {}
